@@ -1,11 +1,14 @@
 import logging
 
 import click
+
 from pymobiledevice3.cli.cli_common import Command
 from pymobiledevice3.lockdown import LockdownClient
 from pymobiledevice3.resources.firmware_notifications import get_notifications
 from pymobiledevice3.services.notification_proxy import NotificationProxyService
 
+logger = logging.getLogger(__name__)
+
 
 @click.group()
 def cli():
@@ -37,7 +40,7 @@ def observe(lockdown: LockdownClient, names):
         service.notify_register_dispatch(name)
 
     for event in service.receive_notification():
-        logging.info(event)
+        logger.info(event)
 
 
 @notification.command('observe-all', cls=Command)
@@ -48,4 +51,4 @@ def observe_all(lockdown: LockdownClient):
         service.notify_register_dispatch(notification)
 
     for event in service.receive_notification():
-        logging.info(event)
+        logger.info(event)
