@@ -6,7 +6,8 @@ import traceback
 import IPython
 import click
 from pygments import highlight, lexers, formatters
-from pymobiledevice3.cli.cli_common import print_json
+
+from pymobiledevice3.cli.cli_common import print_json, set_verbosity
 from pymobiledevice3.exceptions import IncorrectModeError
 from pymobiledevice3.irecv import IRecv
 from pymobiledevice3.lockdown import list_devices, LockdownClient
@@ -20,12 +21,15 @@ SHELL_USAGE = """
 print(irecv.getenv('build-version'))
 """
 
+logger = logging.getLogger(__name__)
+
 
 class Command(click.Command):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.params[:0] = [
             click.Option(('device', '--ecid'), type=click.INT, callback=self.device),
+            click.Option(('verbosity', '-v', '--verbose'), count=True, callback=set_verbosity, expose_value=False),
         ]
 
     @staticmethod
@@ -35,18 +39,18 @@ class Command(click.Command):
             return
 
         ecid = value
-        logging.debug('searching among connected devices via lockdownd')
+        logger.debug('searching among connected devices via lockdownd')
         for udid in list_devices():
             try:
                 lockdown = LockdownClient(udid=udid)
             except IncorrectModeError:
                 continue
             if (ecid is None) or (lockdown.ecid == value):
-                logging.debug('found device')
+                logger.debug('found device')
                 return lockdown
             else:
                 continue
-        logging.debug(f'waiting for device to be available in Recovery mode')
+        logger.debug('waiting for device to be available in Recovery mode')
         return IRecv(ecid=ecid)
 
 
@@ -73,9 +77,10 @@ def restore_shell(device):
 
 
 @restore.command('enter', cls=Command)
-def restore_enter(lockdown):
+def restore_enter(device):
     """ enter Recovery mode """
-    lockdown.enter_recovery()
+    if isinstance(device, LockdownClient):
+        device.enter_recovery()
 
 
 @restore.command('exit')
