@@ -38,6 +38,18 @@ class CrashReport:
                 field = field.strip()
                 return field
 
+    @cached_property
+    def bug_type(self):
+        return self._metadata['bug_type']
+
+    @cached_property
+    def incident_id(self):
+        return self._metadata['incident_id']
+
+    @cached_property
+    def timestamp(self):
+        return self._metadata['timestamp']
+
     @cached_property
     def name(self) -> str:
         return self._metadata['name']
@@ -117,7 +129,7 @@ class CrashReport:
     @cached_property
     def exception_type(self):
         if self._is_json:
-            return self._data['exception']['type']
+            return self._data['exception'].get('type')
         else:
             return self._parse_field('Exception Type')
 
@@ -132,7 +144,7 @@ class CrashReport:
     def application_specific_information(self) -> str:
         result = ''
         if self._is_json:
-            pass
+            return str(self._data.get('asi'))
         else:
             in_frames = False
             for line in self._data.split('\n'):
@@ -143,7 +155,7 @@ class CrashReport:
 
                     result += line + '\n'
 
-                if line.startswith(f'Application Specific Information:'):
+                if line.startswith('Application Specific Information:'):
                     in_frames = True
 
         return result
@@ -151,6 +163,12 @@ class CrashReport:
     def __str__(self):
         result = ''
 
+        result += click.style(f'{self.incident_id} {self.timestamp}\n\n', fg='cyan')
+
+        if self.bug_type not in ('109', '309', '327', '385', ):
+            # these crashes aren't crash dumps
+            return result
+
         result += click.style(f'Exception: {self.exception_type}\n', bold=True)
 
         if self.exception_subtype:
@@ -161,10 +179,6 @@ class CrashReport:
             result += click.style('Application Specific Information: ', bold=True)
             result += self.application_specific_information
 
-        if self._is_json:
-            if self._data['asi']:
-                result += click.style('ASI: ', bold=True) + str(self._data['asi']) + '\n'
-
         result += '\n'
 
         result += click.style('Registers:', bold=True)
@@ -196,7 +210,8 @@ class CrashReports:
         """
         Clear all crash reports.
         """
-        self.afc.rm('/', force=True)
+        for filename in self.ls('/'):
+            self.afc.rm(filename, force=True)
 
     def ls(self, path: str = '/', depth: int = 1):
         """
@@ -217,7 +232,7 @@ class CrashReports:
         Path(out).mkdir(exist_ok=True, parents=True)
 
         def log(src, dst):
-            logging.info(f'{src} --> {dst}')
+            self.logger.info(f'{src} --> {dst}')
 
         self.afc.pull(entry, out, callback=log)
 
@@ -244,6 +259,11 @@ class CrashReports:
                 continue
 
             filename = posixpath.basename(syslog_entry.message.split()[-1])
+            self.logger.debug(f'crash report: {filename}')
+
+            if posixpath.splitext(filename)[-1] not in ('.ips', '.panic'):
+                continue
+
             crash_report_raw = self.afc.get_file_contents(filename).decode()
             crash_report = CrashReport(crash_report_raw)
 
